<?php

if (!defined('ABSPATH')) {
	exit;
}

class WC_HC_Communicator {
	public $last_error = null;

	public function load_all_users() {
		$all_users = [];

		$load_more = false;
		$offset = 1;
		do {
			$users = $this->perform_api_request('users?limit=250&offset='.$offset);
			if (!empty($users['error'])) {
				$this->last_error = $users['error'];
				return [];
			}
			if ($users['users']) {
				$all_users = array_merge($users['users'], $all_users);
			}
			$load_more = $users['users'] && count($users['users']) == 250;
			$offset++;
		} while ($load_more);

		return $all_users;
	}

	public function create_customer($order) {
		$data = $this->_customer_data_from_order($order);
		return $this->perform_api_request('users', 'POST', $data);
	}

	public function update_customer($user, $order) {
		$data = $this->_customer_data_from_order($order);
		$data['user_id'] = $user['user_id'];
		return $this->perform_api_request('users', 'POST', $data);
	}

	public function create_invoice($order, $user) {
		$items = [];

		// Products
		foreach ($order->get_items() as $item_id => $item) {
			$product = $item->get_product();
			$tax_rate = $this->_get_tax_rate_for_item($order, $item);

			// Subtotal (=before discount)
			$subtotal_price_inc_tax = $order->get_line_subtotal($item, true, true);
			$subtotal_single_price = $subtotal_price_inc_tax / $item['qty'];

			// Total (=after discount)
			$total_price_inc_tax = $order->get_line_total($item, true, true);

			// Filter Item Name
			$name = $item['name'];
			$name = apply_filters('e5_hc_create_invoice_item_name', $name, $item, $product);

			// item_price is calculated for one single product including tax
			// item_discount_value is calculated by using the difference between subtotal and total
			$item_to_add = [
				'item_name' => $name,
				'item_quantity' => $item['qty'],
				'item_price' => $subtotal_single_price,
				'item_taxRate' => $tax_rate,
				'item_discount_unit' => 'absolute',
				'item_discount_value' => $subtotal_price_inc_tax - $total_price_inc_tax,
			];

			// Add item_article_id (if known)
			$hc_article_id = $this->_hc_article_id_for_order_item($item);
			if (!empty($hc_article_id)) {
				$item_to_add['item_article_id'] = $hc_article_id;
			}

			$items[] = $item_to_add;
		}

		// Shipping
		foreach ($order->get_shipping_methods() as $shipping_item_id => $shipping_item) {
			$tax_rate = $this->_get_tax_rate_for_item($order, $shipping_item);

			$items[] = [
				'item_name' => $shipping_item['name'],
				'item_quantity' => 1,
				'item_price' => $order->get_line_total($shipping_item, true, true),
				'item_taxRate' => $tax_rate,
				'item_discount_unit' => 'absolute',
				'item_discount_value' => 0
			];
		}

		// Fees
		foreach ($order->get_fees() as $fee_item_id => $fee_item) {
			$tax_rate = $this->_get_tax_rate_for_item($order, $fee_item);

			$items[] = [
				'item_name' => $fee_item['name'],
				'item_quantity' => 1,
				'item_price' => $order->get_line_total($fee_item, true, true),
				'item_taxRate' => $tax_rate,
				'item_discount_unit' => 'absolute',
				'item_discount_value' => 0
			];
		}

		// Coupons
		foreach ($order->get_coupons() as $coupon_item_id => $coupon_item) {
			$tax_rate = $this->_get_tax_rate_for_item($order, $coupon_item);

			$items[] = [
				'item_name' => $coupon_item['name'],
				'item_quantity' => 1,
				'item_price' => 0 - $order->get_line_total($coupon_item, true, true),
				'item_taxRate' => $tax_rate,
				'item_discount_unit' => 'absolute',
				'item_discount_value' => 0
			];
		}

		// Payment Method
		$paymentMethod = $order->get_payment_method();
		$optionMethod = get_option('e5_hc_payment_gateway_'.$paymentMethod);
		$loadedMethods = $this->load_payment_methods();
		foreach ($loadedMethods as $loadedMethod) {
			if ($loadedMethod['paymentMethod_id'] == $optionMethod) {
				$paymentMethod = $loadedMethod['paymentMethod_name'];
			}
		}

		// Cashier Id
		$cashier_id = get_option('e5_hc_employee');

		// Test mode
		$testMode = false;
		$testModeOption = get_option('e5_hc_test_mode');
		if (isset($testModeOption) && $testModeOption == true && $testModeOption != 'no') {
			$testMode = true;
		}

		// Filter invoice text
		$invoice_text = apply_filters('e5_hc_invoice_text', '', $order, $paymentMethod);

		$data = [
			'cashier_id' => $cashier_id,
			'invoice_testMode' => $testMode,
			'invoice_text' => $invoice_text,
			'invoice_reference' => 0,
			'invoice_paymentMethod' => $paymentMethod,
			'invoice_discount_percent' => 0,
			'invoice_user_id' => $user['user_id'],
			'signature_mandatory' => 0,
			'items' => $items
		];

		return $this->perform_api_request('invoices', 'POST', $data);
	}


	public function cancel_invoice($invoice_id, $reason) {
		$cashier_id = get_option('e5_hc_employee');

		$data = [
			'cancellation_cashier_id' => $cashier_id,
			'cancellation_reason' => $reason,
			'cancellation_cashBook_entry' => false,
		];

		return $this->perform_api_request('invoices/'.$invoice_id.'/cancellation', 'POST', $data);
	}

	public function decrease_stock($article_id, $qty, $invoice_number = '') {
		$data = [
			'stock_change' => 0 - $qty,
			'stock_invoice_number' => ''.$invoice_number,
			'stock_delivery_note_number' => '',
			'stock_description' => ''
		];

		return $this->perform_api_request('articles/'.$article_id.'/change-stock', 'POST', $data);
	}

	public function increase_stock($article_id, $qty, $invoice_number = '') {
		$data = [
			'stock_change' => $qty,
			'stock_invoice_number' => ''.$invoice_number,
			'stock_delivery_note_number' => '',
			'stock_description' => ''
		];

		return $this->perform_api_request('articles/'.$article_id.'/change-stock', 'POST', $data);
	}

	public function get_pdf_invoice($invoice_id) {
		$locale = 'de_AT';

		// Get locale setting
		$locale_setting = get_option('e5_hc_locale');
		if (!empty($locale_setting)) {
			$locale = $locale_setting;
		}

		$result = $this->perform_api_request('invoices/'.$invoice_id.'/pdf?locale=' . $locale);
		if (!empty($result['error'])) {
			$this->last_error = $result['error'];
			return [];
		}
		return $result;
	}

	public function load_employees() {
		$result = $this->perform_api_request('employees');
		if (!empty($result['error'])) {
			$this->last_error = $result['error'];
			return [];
		}
		return $result;
	}

	public function load_payment_methods() {
		$result = $this->perform_api_request('paymentMethods');
		if (!empty($result['error'])) {
			$this->last_error = $result['error'];
			return [];
		}
		return $result;
	}

	public function load_all_articles() {
		$all_articles = [];

		$load_more = false;
		$offset = 1;
		do {
			$articles = $this->perform_api_request('articles?limit=250&offset='.$offset);
			if (!empty($articles['error'])) {
				$this->last_error = $articles['error'];
				return [];
			}

			if ($articles['articles']) {
				$all_articles = array_merge($articles['articles'], $all_articles);
			}

			$load_more = $articles['articles'] && count($articles['articles']) == 250;
			$offset++;
		} while ($load_more);

		return $all_articles;
	}

	public function load_article($article_id) {
		$result = $this->perform_api_request('articles/'.$article_id);
		if (!empty($result['error'])) {
			$this->last_error = $result['error'];
			return [];
		}
		return $result;
	}

	public function create_article($product) {
		$product_id = $product->get_id();
		$name = $product->get_name();

		if ($product->get_type() === 'variation') {
			$product_id = $product->get_parent_id();
			$name = $product->get_formatted_name();
		}

		$name = apply_filters('e5_hc_create_article_name', $name, $product);

		// Find category (first one we find that has a mapping)
		$category_id = 0;

		$categories = get_the_terms($product_id, 'product_cat');
		foreach ($categories as $category) {
			$hc_category_id = get_option('e5_hc_category_map_'.$category->term_id);
			if (!empty($hc_category_id)) {
				$category_id = $hc_category_id;
				break;
			}
		}

		// Find tax rate
		$tax_class = $product->get_tax_class();
		$rates = WC_Tax::get_rates($tax_class);
		$tax_rate = 0;
		foreach ($rates as $a_rate) {
			$tax_rate = floatval($a_rate['rate']);
		}

		$data = [
			'article_name' => $name,
			'article_comment' => $product->get_description(),
			'article_code' => $product->get_sku(),
			'article_category_id' => $category_id,
			'article_tax_rate' => $tax_rate,
			'article_gross_selling_price' => floatval($product->get_price()),
			'article_stock' => $product->get_stock_quantity(),
		];

		// Add attachment image
		if ($product->get_image_id()) {
			try {
				$attachment_path = $this->_scaled_image_path($product->get_image_id());
				if (!empty($attachment_path)) {
					$article_image = file_get_contents($attachment_path);
					$data['article_image'] = base64_encode($article_image);
					$data['article_image_text'] = true;
				}
			} catch (Throwable $t) {
				// Silently ignore error while adding image
			}
		}

		return $this->perform_api_request('articles', 'POST', $data);
	}

	private function perform_api_request($call, $method = 'GET', $data = null) {
		$this->last_error = null;

		$authHeader = '';

		if (!empty(get_option('e5_hc_api_token'))) {
			$authHeader = 'Authorization: Bearer ' . get_option('e5_hc_api_token');
		}

		if (empty($authHeader)) {
			$this->last_error = 'No Authentication provided';
			return [];
		}

		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, 'https://api.hellocash.business/api/v1/' . $call);
		curl_setopt($ch, CURLOPT_HTTPHEADER,
			[
				$authHeader,
				'Content-Type: application/json; charset=utf-8',
				'Accept: application/json'
			]
		);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, true);
		curl_setopt($ch, CURLOPT_TIMEOUT, 10);

		if ($data != null && ($method == 'POST' || $method == 'PUT')) {
			$data = json_encode($data, JSON_UNESCAPED_UNICODE);
			curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
		}

		if ($method == 'POST') {
			curl_setopt($ch, CURLOPT_POST, 1);
		} else if ($method == 'PUT') {
			curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'PUT');
		}

		$response = curl_exec($ch);

		if (curl_errno($ch)) {
			$this->last_error = curl_error($ch);
			return [];
		}

		$result_array = json_decode($response, true);
		curl_close($ch);

		return $result_array;
	}

	private function _get_countries() {
		// we switch locale to get country name in english
		$oldLocale = get_locale();
		switch_to_locale('en_US');

		$path = WP_PLUGIN_DIR . '/woocommerce/i18n/countries.php';
		$countries = apply_filters('woocommerce_countries', include $path);

		// switch back to original locale
		switch_to_locale($oldLocale);

		return $countries;
	}

	private function _get_tax_rate_for_item($order, $item) {
		$tax_rate = 0.0;

		if ($order->get_line_tax( $item ) > 0.0) {
			$item_data = $item->get_data();
			$item_tax  = [];
			$rate_id = false;

			if (isset($item_data['taxes']['subtotal'])) {
				$item_tax = $item_data['taxes']['subtotal'];
			} else if (isset($item_data['taxes']['total'])) {
				$item_tax = $item_data['taxes']['total'];
			}

			if (!empty($item_tax)) {
				foreach ($item_tax as $maybe_rate_id => $tax_amount) {

					if (empty($tax_amount)) {
						continue;
					}

					$rate_id = $maybe_rate_id;
					break;
				}
			}

			if ($rate_id) {
				$tax_rate = WC_Tax::get_rate_percent_value($rate_id);
			} else {
				$tax_rate = $order->get_line_tax($item) / $order->get_line_subtotal($item, false) * 100;
			}
		}

		return $tax_rate;
	}

	private function _customer_data_from_order($order) {
		$data = [
			'user_firstname' => $order->get_billing_first_name(),
			'user_surname' => $order->get_billing_last_name(),
			'user_street' => $order->get_billing_address_1() . $order->get_billing_address_2(),
			'user_city' => $order->get_billing_city(),
			'user_postalCode' => $order->get_billing_postcode(),
			'user_country' => $this->_get_countries()[$order->get_billing_country()],
			'user_email' => $order->get_billing_email(),
			'user_phoneNumber' => $order->get_billing_phone()
		];

		$company_name = $order->get_billing_company();
		if (!empty($company_name)) {
			$data['user_company'] = $company_name;
		}

		$keys = ['_billing_vat_id', 'billing_vat'];
		foreach ($keys as $key) {
			$billing_vat = get_post_meta($order->get_id(), $key, true);
			if ($billing_vat && !empty($billing_vat)) {
				$data['user_uidNumber'] = $billing_vat;
				break;
			}
		}

		return $data;
	}

	private function _scaled_image_path($attachment_id, $size = 'thumbnail') {
		$file = get_attached_file($attachment_id, true);
		if (empty($size) || $size === 'full') {
			return realpath($file);
		}
		if (! wp_attachment_is_image($attachment_id) ) {
			return false;
		}
		$info = image_get_intermediate_size($attachment_id, $size);
		if (!is_array($info) || ! isset($info['file'])) {
			return false;
		}

		return realpath(str_replace(wp_basename($file), $info['file'], $file));
	}

	private function _hc_article_id_for_order_item($item) {
		$product = $item->get_product();

		// dont handle products with children (they get handlet separately later on)
		if (!empty($product->get_children())) {
			return null;
		}

		$article_data = get_post_meta($product->get_id(), '_e5_hc_article_id', true);

		if (!empty($article_data)) {
			$article_data = json_decode($article_data, true);
			if (!empty($article_data['id'])) {
				return $article_data['id'];
			}
		}
		
		return null;
	}
}
