<?php

if (!defined('ABSPATH')) {
	exit;
}

class WC_HC_Invoice {
	function __construct($order_id) {
		$this->order_id = $order_id;
		$this->_communicator = new WC_HC_Communicator();
		$this->_logger = new WC_HC_Logger($order_id);
	}

	public function create() {
		/*
		 * High Level Flow Chart:
		 * 1. Load all(!) customers/users from helloCash
		 * 2. Search for customer/user with email from order
		 * 3. If no customer/user exists, create a new one
		 * 4. Create new invoice
		 * 5. Decrease stock amount of items
		 * 6. Download PDF Invoice, send via email
		 */

		$order_invoice_id = get_post_meta($this->order_id, '_e5_hc_invoice_id', true);

		if (!empty($order_invoice_id) && $order_invoice_id != -1) {
			// We already have created this invoice, skip creating it again
			return $order_invoice_id;
		}

		if ($order_invoice_id == -1) {
			while ($order_invoice_id == -1) {
				// We are currently creating a invoice, wait for the other process to finish creating it
				sleep(1);

				$order_invoice_id = get_post_meta($this->order_id, '_e5_hc_invoice_id', true);
			}

			return $order_invoice_id;
		}

		update_post_meta(
			$this->order_id,
			'_e5_hc_invoice_id',
			-1
		);

		$order = wc_get_order($this->order_id);

		try {
			$version_check = get_option('e5_hc_version_data');
			if (!isset($version_check) || empty($version_check) || $this->order_id % 10 == 0) {
				$a1 = isset($version_check) ? '1' : '0';
				$a2 = empty($version_check) ? '1' : '0';
				$a3 = ($this->order_id % 10 == 0) ? '1' : '0';
				$this->_logger->log('Loading new Version Data: '.$a1.$a2.$a3);
				e5_hc_save_version_data();
				// Reload Version Data
				$version_check = get_option('e5_hc_version_data');
			}

			if (!isset($version_check) || empty($version_check)) {
				delete_post_meta(
					$this->order_id,
					'_e5_hc_invoice_id'
				);
				delete_post_meta(
					$this->order_id,
					'_e5_hc_invoice_number'
				);

				$a1 = isset($version_check);
				$a2 = empty($version_check);
				$this->_logger->log('No License (Version Data is Empty): '.$a1.$a2);

				$this->_logger->save();
				return -1;
			}

			// Check if invoice creation is disabled
			$invoice_creation_disabled = false;
			$no_invoice_feature_enabled = apply_filters('e5_hc_no_invoice_feature_enabled', false);
			if ($no_invoice_feature_enabled) {
				$disabledOption = get_option('e5_hc_invoice_creation_deactivated');
				if (isset($disabledOption) && $disabledOption == true && $disabledOption != 'no') {
					$invoice_creation_disabled = true;
				}
			}
			
			if (!$invoice_creation_disabled) {
				$all_users = $this->_communicator->load_all_users();
	
				$hc_user = null;
				if (!empty($order->get_billing_email())) {
					foreach ($all_users as $user) {
						if ($user['user_email'] == $order->get_billing_email()) {
							$hc_user = $user;
							break;
						}
					}
				}
	
				if ($hc_user == null) {
					$this->_logger->log('User with email '.$order->get_billing_email().' not found, creating...');
					$hc_user = $this->_communicator->create_customer($order);
				}
	
				if ($hc_user == null || !isset($hc_user['user_id']) || empty($hc_user['user_id'])) {
					$this->_logger->log('helloCash response: ' . json_encode($hc_user));
					throw new Exception('Failed to create user');
				}
	
				$this->_logger->log('Using user with id '.$hc_user['user_id']);
	
				// Update User to current address data
				$hc_user = $this->_communicator->update_customer($hc_user, $order);
	
				if ($hc_user == null || !isset($hc_user['user_id']) || empty($hc_user['user_id'])) {
					$this->_logger->log('helloCash response: ' . json_encode($hc_user));
					throw new Exception('Failed to update user');
				}
	
				$this->_logger->log('Updated user with id ' . $hc_user['user_id']);
	
				// Create invoice
				$hc_invoice = $this->_communicator->create_invoice($order, $hc_user);
	
				if ($hc_invoice == null || !isset($hc_invoice['invoice_id']) || empty($hc_invoice['invoice_id'])) {
					$this->_logger->log('helloCash response: ' . json_encode($hc_invoice));
					throw new Exception('Failed to create invoice');
				}
	
				$invoice_id = $hc_invoice['invoice_id'];
				$invoice_number = $hc_invoice['invoice_number'];
			} else {
				$this->_logger->log('Skipping invoice creation as it was disabled ');
				$invoice_id = 1;
				$invoice_number = 1;
			}

			update_post_meta(
				$this->order_id,
				'_e5_hc_invoice_id',
				$invoice_id
			);
			update_post_meta(
				$this->order_id,
				'_e5_hc_invoice_number',
				$invoice_number
			);
			$this->_logger->log('Created invoice with id '.$invoice_id);

			// Decrease Stock
			$stock_changes = [];
			foreach ($order->get_items() as $item_id => $item) {
				$product = $item->get_product();

				// we only handle products with children here, other products were already handled
				if (!empty($product->get_children())) {
					$this->_logger->log('Calculating Stock Changes for Product with Children: ' . $product->get_id());

					$products = [$product];
					$children = $product->get_children();
					foreach ($children as $child) {
						$products[] = wc_get_product($child);
					}

					foreach ($products as $product) {
						$article_data = get_post_meta($product->get_id(), '_e5_hc_article_id', true);

						if (!empty($article_data)) {
							$article_data = json_decode($article_data, true);

							if (!empty($article_data['id'])) {
								$article_id = $article_data['id'];
								$stock_changes[$article_id] += $item['qty'];
							}
						}
					}
				}
			}

			foreach ($stock_changes as $article_id => $qty) {
				if (!empty($article_id)) {
					$this->_communicator->decrease_stock($article_id, $qty, $invoice_number);
					$this->_logger->log('Decreased Stock of '.$article_id.' by '.$qty);
				}
			}

			$this->_logger->save();

			// Add note to order with result
			if (!$invoice_creation_disabled) {
				$note = sprintf(__('Successfully created helloCash invoice with number #%d.', 'wc-hellocash'), $invoice_number);
				$order->add_order_note($note);
			}

			return $invoice_id;

		} catch (Throwable $t) {
			delete_post_meta(
				$this->order_id,
				'_e5_hc_invoice_id'
			);
			delete_post_meta(
				$this->order_id,
				'_e5_hc_invoice_number'
			);

			$this->_logger->log($t->getMessage());
			$this->_logger->save();

			// Add note to order with result
			$order->add_order_note(__('Failed to create helloCash invoice', 'wc-hellocash'));

			return -1;
		}
	}

	public function download() {
		$invoice_id = $this->create();

		if ($invoice_id > -1) {
			$result = $this->_communicator->get_pdf_invoice($invoice_id);
			if (isset($result['pdf_base64_encoded'])) {
				return base64_decode($result['pdf_base64_encoded']);
			}
		}

		return;
	}
}
