<?php

if (!defined('ABSPATH')) {
	exit;
}

class WC_HC_Stock_Synchronizer {
	private $_enabled = false;
	private $_priceEnabled = false;

	function __construct() {
		$this->_communicator = new WC_HC_Communicator();
		$this->_logger = new WC_HC_Logger('_e5_hc_stock_synchronizer_log');

		// Check if enabled
		$enabledOption = get_option('e5_hc_stock_sync_enable');
		if (isset($enabledOption) && $enabledOption == true && $enabledOption != 'no') {
			$this->_enabled = true;
		}

		// Check if price syncing is enabled
		$enabledOption = get_option('e5_hc_price_sync_enable');
		if (isset($enabledOption) && $enabledOption == true && $enabledOption != 'no') {
			$this->_priceEnabled = true;
		}

		if ($this->_enabled) {
			// Register custom Cron Schedule
			add_filter('cron_schedules', [$this, 'add_custom_cron_schedules']);

			// Schedule Cron if necessary
			add_action('init', [$this, 'init']);

			// Register Cron method
			add_action('e5_hc_stock_synchronizer_cron', [$this, 'run_cron']);
		}
	}

	function add_custom_cron_schedules($schedules) {
		$schedules['e5-hc-every-5-minutes'] = [
			'interval' => 5 * MINUTE_IN_SECONDS,
			'display' => __('Every 5 minutes'),
		];
		return $schedules;
	}

	function init() {
		if (!wp_next_scheduled('e5_hc_stock_synchronizer_cron')) {
			wp_schedule_event(time(), 'e5-hc-every-5-minutes', 'e5_hc_stock_synchronizer_cron');
		}
	}

	function run_cron() {
		if (!$this->_enabled) {
			// Cron job is still running but syncing is actually disabled
			$this->_logger->log("Skipping Stock/Price Sync (disabled)");
			return;
		}

		$this->_logger->log("Starting Stock/Price sync");

		try {
			// Actually run the import
			$this->_import_stock_from_hellocash();
		} catch (Throwable $t) {
			$this->_logger->log($t->getMessage());
		}

		// Store last run timestamp
		update_option('_e5_hc_stock_synchronizer_last_run', time());

		// Save Log
		$this->_logger->log("Finished Stock/Price sync");
		$this->_logger->save();
	}

	private function _import_stock_from_hellocash() {
		// Load all products from helloCash
		$hc_products = $this->_communicator->load_all_articles();
		$this->_logger->log("Loaded " . count($hc_products) . " helloCash products.");

		// Load all products from wooCommerce
		$products = wc_get_products([
			'type' => [
				'simple',
				'grouped',
				'external',
				'variation'
			],
			'posts_per_page' => -1,
		]);
		$this->_logger->log("Loaded " . count($products) . " wooCommerce products.");

		// Go over each WooCommerce Product
		foreach ($products as $product) {
			try {
				// Go over each wooCommerce product
				$this->_logger->log("Processing WooCommerce product: " . $product->get_id() . ": " . $product->get_name());

				// Find associated helloCash product
				$product_hc_article = get_post_meta($product->get_id(), '_e5_hc_article_id', true);

				if (!empty($product_hc_article)) {
					$value_decoded = json_decode($product_hc_article, true);
					// $product_hc_article_id = the helloCash article ID saved as meta to the WooCommerce product
					$product_hc_article_id = $value_decoded['id'];

					$stock_retrieved = false;

					// Find the helloCash product in all loaded helloCash products
					foreach ($hc_products as $hc_product) {
						if ($hc_product['article_id'] === $product_hc_article_id) {
							// Found helloCash product
							$stock_retrieved = true;
							$needs_save = false;

							// Update price if needed
							$hc_price = $hc_product['article_gross_sellingPrice'];
							if ($this->_priceEnabled && $product->get_regular_price() != $hc_price) {
								$this->_logger->log("Changing price from: " . $product->get_regular_price() . " to: " . $hc_price);

								$product->set_regular_price($hc_price);
								$needs_save = true;
							}

							// Update stock
							$hc_stock = intval($hc_product['article_stock']);
							if ($product->get_manage_stock() === true && $product->get_stock_quantity() != $hc_stock) {
								$this->_logger->log("Changing stock from: " . $product->get_stock_quantity() . " to: " . $hc_stock);

								$product->set_stock_quantity($hc_stock);
								$needs_save = true;
							} else if ($product->get_manage_stock() !== true) {
								$this->_logger->log("Manage stock disabled for woocommerce product");
							}

							// Save if required
							if ($needs_save) {
								$product->save();
							}

							break;
						}
					}

					if (!$stock_retrieved) {
						// No article with the ID was found or article not linked
						$this->_logger->log("Did not find helloCash product for this wooCommerce product, resetting stock to 0");
					}
				} else {
					// No helloCash article was associated with this WooCommerce Product
					$this->_logger->log("No helloCash article was linked with this WooCommerce product.");
				}
			} catch (Throwable $t) {
				$this->_logger->log($t->getMessage());
			}
		}
	}
}
