<?php

if (!defined('ABSPATH')) {
	exit;
}

if (class_exists('WC_Settings_HelloCash', false)) {
	return new WC_Settings_HelloCash();
}

class WC_Settings_HelloCash extends WC_Settings_Page {
	public function __construct() {
		$this->id    = 'hellocash';
		$this->label = __('helloCash', 'wc-hellocash');

		$this->employees = [];
		$this->paymentMethods = [];

		$this->_communicator = new WC_HC_Communicator();

		add_action('woocommerce_settings_start', [$this, 'check_login_credentials']);
		add_action('admin_notices', [$this, 'show_admin_notices']);

		parent::__construct();
	}

	public function check_login_credentials() {
		global $current_tab;

		// Check for Errors
		if (!$this->should_show_version_check() && $current_tab === 'hellocash') {
			$this->load_hellocash_elements();
			if ($this->_communicator->last_error) {
				WC_Admin_Settings::add_error('helloCash: ' . $this->_communicator->last_error);
			}
		}
	}

	public function show_admin_notices() {
		if ($this->_communicator->last_error) {
			?>
			<div class="notice notice-error">
				<p><?php echo 'helloCash: ' . $this->_communicator->last_error; ?></p>
			</div>
			<?php
		}
	}

	protected function should_show_version_check() {
		$version_check = get_option('e5_hc_version_data');
		if (!isset($version_check) || empty($version_check)) {
			return true;
		}

		$version_data = json_decode(base64_decode($version_check), true);
		$urlparts = parse_url(home_url());
		$domain = $urlparts['host'];
		$foo = null;
		foreach ($version_data['domains'] as $dom) {
			if (strpos($domain, $dom) !== false) {
				$foo = $dom;
			}
		}

		if ($foo === null) {
			return true;
		}

		return false;
	}

	protected function get_settings_for_default_section() {
		if ($this->should_show_version_check()) {
			return $this->get_activation_settings();
		}

		// General Settings
		$settings = [
			[
				'title' => __('helloCash Credentials', 'wc-hellocash'),
				'type'  => 'title',
				'desc'  => __('These are the credentials used to log into your helloCash account.', 'wc-hellocash'),
				'id'    => 'e5_hc_credentials',
			],
			[
				'title' => __('API Token', 'wc-hellocash'),
				'type'  => 'text',
				'desc'  => __('Your API Token. Get it from helloCash > Cash Register > Settings > General > helloCash API and click on Generate Token.', 'wc-hellocash'),
				'id'    => 'e5_hc_api_token',
				'desc_tip' => true,
			],
			[
				'type' => 'sectionend',
				'id'   => 'e5_hc_credentials',
			]
		];

		// Employee Setting
		$employees = $this->employees;
		if (count($employees) > 0) {
			$options = [];
			foreach ($employees as $employee) {
				$options[$employee['employee_id']] = $employee['employee_name'];
			}

			$settings[] = [
				'title' => __('helloCash Employee', 'wc-hellocash'),
				'type'  => 'title',
				'desc'  => __('Details about the employee used to create invoices', 'wc-hellocash'),
				'id'    => 'e5_hc_employees',
			];
			$settings[] = [
				'title' => __('Employee', 'wc-hellocash'),
				'type'  => 'select',
				'desc'  => __('The employee used to create new invoices.', 'wc-hellocash'),
				'id'    => 'e5_hc_employee',
				'desc_tip' => true,
				'options' => $options,
			];
			$settings[] = [
				'type' => 'sectionend',
				'id'   => 'e5_hc_employees',
			];

			// Test Mode
			$settings[] = [
				'title' => __('helloCash Test Mode', 'wc-hellocash'),
				'type'  => 'title',
				'desc'  => __('Enable Test Mode to test all settings and create test invoices in helloCash. Test invoices will only show up in helloCash if you also switch to the test mode in helloCash.', 'wc-hellocash'),
				'id'    => 'e5_hc_test_mode_section',
			];
			$settings[] = [
				'title' => __('Test Mode', 'wc-hellocash'),
				'type'  => 'checkbox',
				'id'    => 'e5_hc_test_mode',
				'desc_tip' => true,
			];
			$settings[] = [
				'type' => 'sectionend',
				'id'   => 'e5_hc_test_mode_section',
			];
		}

		// Invoice Creation
		$settings[] = [
			'title' => __('Invoice Creation', 'wc-hellocash'),
			'type'  => 'title',
			'desc'  => __('Decide if invoices should be generated when an order is processing or completed.', 'wc-hellocash'),
			'id'    => 'e5_hc_invoice_creation_section',
		];
		$settings[] = [
			'title' => __('Completed', 'wc-hellocash'),
			'type'  => 'checkbox',
			'desc'  => __('When checked, invoices will be created after orders are completed.', 'wc-hellocash'),
			'id'    => 'e5_hc_invoice_creation_completed',
			'desc_tip' => true,
		];
		
		$no_invoice_feature_enabled = apply_filters('e5_hc_no_invoice_feature_enabled', false);
		if ($no_invoice_feature_enabled) {
			$settings[] = [
				'title' => __('Deactivated', 'wc-hellocash'),
				'type'  => 'checkbox',
				'desc'  => __('When checked, no invoices will be created.', 'wc-hellocash'),
				'id'    => 'e5_hc_invoice_creation_deactivated',
				'desc_tip' => true,
			];
		}
		
		// Language of PDF Invoice
		$settings[] = [
			'title' => __('Locale', 'wc-hellocash'),
			'type'  => 'select',
			'desc'  => __('Locale (language) used in PDF invoices.', 'wc-hellocash'),
			'id'    => 'e5_hc_locale',
			'desc_tip' => true,
			'options' => [
				'de_AT' => __('Austria', 'wc-hellocash'),
				'de_DE' => __('Germany', 'wc-hellocash'),
				'en_GB' => __('Britain', 'wc-hellocash'),
				'es_ES' => __('Spain', 'wc-hellocash'),
				'fr_FR' => __('France', 'wc-hellocash')
			],
		];

		// Check if warning should be displayed
		$show_email_warning = null;

		foreach(['customer_processing_order', 'customer_completed_order'] as $email_id) {
			$wc_settings = get_option('woocommerce_'.$email_id.'_settings');
			if (isset($wc_settings['e5_hc_attach_hellocash_pdf']) && ($wc_settings['e5_hc_attach_hellocash_pdf'] == true && $wc_settings['e5_hc_attach_hellocash_pdf'] != 'no')) {
				$show_email_warning = $email_id;
				break;
			}
		}

		if ($show_email_warning !== null) {
			$settings[] = [
				'type'  => 'title',
				'desc'  => sprintf(__('Because of the %s email, invoices might be generated before being Processing or Completed. If you don\'t want this, disable attaching the invoice to this email and send the invoice PDF manually.', 'wc-hellocash'), $show_email_warning),
				'id'    => 'e5_hc_invoice_creation_section',
			];
		}

		$settings[] = [
			'type' => 'sectionend',
			'id'   => 'e5_hc_invoice_creation_section',
		];

		// Cancellation Creation
		$settings[] = [
			'title' => __('Refund Creation', 'wc-hellocash'),
			'type'  => 'title',
			'desc'  => __('Decide if cancellations should be automatically created.', 'wc-hellocash'),
			'id'    => 'e5_hc_cancellation_creation_section',
		];
		$settings[] = [
			'title' => __('Disable Cancellation Creation', 'wc-hellocash'),
			'type'  => 'checkbox',
			'desc'  => __('When checked, cancellations will not be created.', 'wc-hellocash'),
			'id'    => 'e5_hc_cancellation_creation_disabled',
			'desc_tip' => true,
		];
		$settings[] = [
			'type' => 'sectionend',
			'id'   => 'e5_hc_cancellation_creation_section',
		];


		// Payment Methods Settings
		$paymentMethods = $this->paymentMethods;
		$gateways = WC()->payment_gateways->payment_gateways();

		if (count($paymentMethods) > 0) {
			$options = [];
			foreach ($paymentMethods as $method) {
				$options[$method['paymentMethod_id']] = $method['paymentMethod_name'];
			}

			$settings[] = [
				'title' => __('helloCash Payment Methods', 'wc-hellocash'),
				'type'  => 'title',
				'desc'  => __('Setup mapping between WooCommerce payment gateways and helloCash payment methods.', 'wc-hellocash'),
				'id'    => 'e5_hc_payment_methods',
			];

			foreach ($gateways as $gateway_id => $gateway) {
				$title = $gateway->title;
				if ($gateway->enabled != 'yes') {
					$title .= ' ' . __('(inactive)', 'wc-hellocash');
				}

				$settings[] = [
					'title' => $title,
					'type'  => 'select',
					'id'    => 'e5_hc_payment_gateway_'.$gateway_id,
					'options' => $options,
				];
			}

			$settings[] = [
				'type' => 'sectionend',
				'id'   => 'e5_hc_payment_methods',
			];
		}

		// Sync Stock Amount Settings
		$settings[] = [
			'title' => __('Sync helloCash Stock', 'wc-hellocash'),
			'type'  => 'title',
			'desc'  => __('Configure if helloCash for WooCommerce should sync the available stock from helloCash to WooCommerce', 'wc-hellocash'),
			'id'    => 'e5_hc_stock_sync',
		];
		$settings[] = [
			'title' => __('Sync Stock', 'wc-hellocash'),
			'type'  => 'checkbox',
			'id'    => 'e5_hc_stock_sync_enable',
			'desc_tip' => true,
			'desc'  => __('Sync Stock from helloCash to WooCommerce products', 'wc-hellocash'),
		];
		$settings[] = [
			'title' => __('Sync Price', 'wc-hellocash'),
			'type'  => 'checkbox',
			'id'    => 'e5_hc_price_sync_enable',
			'desc_tip' => true,
			'desc'  => __('Sync Price from helloCash to WooCommerce products', 'wc-hellocash'),
		];
		$settings[] = [
			'type' => 'sectionend',
			'id'   => 'e5_hc_stock_sync',
		];

		// Automatically create new helloCash articles
		$articles_feature_enabled = apply_filters('e5_hc_article_creation_feature_enabled', false);
		if ($articles_feature_enabled) {
			$settings[] = [
				'title' => __('Automatically create new Articles', 'wc-hellocash'),
				'type'  => 'title',
				'desc'  => __('Configure if helloCash for WooCommerce should automatically create new helloCash articles when new WooCommerce products are saved for the first time.', 'wc-hellocash'),
				'id'    => 'e5_hc_auto_create_articles',
			];
			$settings[] = [
				'title' => __('Auto create new Articles', 'wc-hellocash'),
				'type'  => 'checkbox',
				'id'    => 'e5_hc_auto_create_articles_enable',
				'desc_tip' => true,
				'desc'  => __('Automatically create new helloCash articles for new Woocommerce products', 'wc-hellocash'),
			];
			$settings[] = [
				'type' => 'sectionend',
				'id'   => 'e5_hc_auto_create_articles',
			];
		}
		
		// Download Debug Logs
		$settings[] = [
			'title' => __('Debug Logs', 'wc-hellocash'),
			'type'  => 'title',
			'desc'  => '<a href="'.admin_url('/admin-ajax.php?action=e5_hc_download_debug_logs').'" target="_blank" class="button">'.__('Download', 'wc-hellocash').'</a>',
			'id'    => 'e5_hc_download_debug_logs',
		];
		$settings[] = [
			'type' => 'sectionend',
			'id'   => 'e5_hc_download_debug_logs',
		];

		// Activation Settings
		$activation_settings = $this->get_activation_settings();

		return array_merge($settings, $activation_settings);
	}

	private function get_activation_settings() {
		return [
			[
				'title' => __('helloCash for WooCommerce', 'wc-hellocash'),
				'type'  => 'title',
				'desc'  => __('In order to use helloCash for WooCommerce please activate the plugin.', 'wc-hellocash'),
				'id'    => 'e5_hc_activation',
			],
			[
				'title' => __('License Code', 'wc-hellocash'),
				'type'  => 'text',
				'desc'  => __('The License code you received when you bought the plugin.', 'wc-hellocash'),
				'id'    => 'e5_hc_license_code',
				'desc_tip' => true,
			],
			[
				'type' => 'sectionend',
				'id'   => 'e5_hc_activation',
			]
		];
	}

	private function load_hellocash_elements() {
		try {
			$this->employees = $this->_communicator->load_employees();
		} catch (Throwable $t) {
			// TODO
		}

		try {
			$this->paymentMethods = $this->_communicator->load_payment_methods();
		} catch (Throwable $t) {
			// TODO
		}

	}

	public function save() {
		$this->load_hellocash_elements();
		parent::save();
		e5_hc_save_version_data();

		// Ignore helloCash Errors while saving
		$this->_communicator->last_error = null;
	}
}

return new WC_Settings_HelloCash();
